#include "gridstore.h"
#include <stdbool.h>
#ifndef GRIDSTOREFUNCTION_H
#define GRIDSTOREFUNCTION_H

//default GridDB connection settings if not specified by the user
#define HOST "239.0.0.1"
#define PORT "31999"
#define CLUSTER_NAME "defaultCluster"
#define USER "admin"
#define PASSWORD "admin"

#define DEFAULT_TEMPERATURE 180.1//In Fahrenheit
#define DEFAULT_PRESSURE 1.7 // In Bar

// Event States
#define SUCCESS 1
#define CLEAN 2
#define FAILURE 0

// Default values for shot rate and failure rate if the user does not specify them
#define SAMPLE_AVG_SHOTS 100 
#define SAMPLE_FAIL_RATE 0.01
#define SAMPLE_DEVIATION_BETW_SHOTS 12.5

// Default temperature and pressure standard deviation values if the user does not specify them. 
#define SIGMA 4.0// Default standard deviation of drinking water temperature (in Fahrenheit)
#define PRESSURE_SIGMA 0.2 // Estimated std deviation in presure


#define SECONDS_IN_HR 3600

double gaussian_number(double mean, double sigma,unsigned int seed); //Generate a normally / gaussian distributed random number
unsigned int determineEspressoSuccess(double failureProbability,unsigned int seed); // Determines whether an espresso shot SUCCEEDS OR FAILS based on p (probability)
int adjustedShotsPerHour(int averageShots, double standard_deviation,unsigned int seed);
int getSleepTime(int shotCount, int shotRate, int secondsElapsed);
int calculateSecondsPerShot(int averageShot, double standard_deviation, unsigned int seed);
GSGridStore* makeGridStore(char* host, char* port, char* clusterName, char* username, char* password); // Connect  to  a GridDB cluster 
GSCollection* makeCoffeeStoreCollection(GSGridStore* gridstore, char* chainName); //Create a Collection in GridDB that will store all records for Coffee stores, 
GSCollection* makeMachineCollection(GSGridStore* gridstore, char* collectionName); // Create a Collection in GridDB that will store all records for a coffee machine a coffee store
GSTimeSeries* makeEspressoMachine(GSGridStore* gridstore, char* serialNumber); // Creates a TimeSeries that will be used to represent a coffee machine to generate espresso record data .
void insertCoffeeStore(GSCollection* coffeeStoreChain, char* store_id, char* address, float latitude, float longitude); //Insert a coffee store within a coffee chain franchise
void insertCoffeeMachine(GSCollection* coffeeStore, char* serial_number,char* model_number, GSTimestamp installation_date, int cycles_new, int cycles_clean); 
char * determineEventType(int eventState);																			//insert and register a coffee machine into a coffee store
void updateMachineCycles(GSCollection* coffeeStore,char* serialNumber,bool resetCleanCycle);
void makeEspressoRecord(GSTimestamp timestamp,GSTimeSeries* espressoMachine,GSCollection* coffeeStore, char* serialNumber,double average_temperature, double average_pressure, double failure_rate,unsigned int seed,bool isCleanCycle);
#endif